@@ -11,6 +11,7 @@ use crate::client::{Client, Config};
 use crate::types::Keypair;
 use eyre::Result;
 use std::{sync::Once, time::Duration};
+use tempfile::tempdir;
 use tracing_subscriber::{fmt, EnvFilter};
 
 static INIT: Once = Once::new();
@@ -41,10 +42,11 @@ pub async fn create_test_client_with(
     timeout: Option<u64>,
 ) -> Result<Client> {
     init_logger();
+    let root_dir = tempdir().map_err(|e| eyre::eyre!(e.to_string()))?;
     let timeout = timeout.map(Duration::from_secs);
-    let contact_info = read_network_conn_info()?.into_iter().collect();
-    let config = Config::new(None, Some(contact_info), None, timeout).await;
-    let client = Client::new(optional_keypair.clone(), config).await?;
+    let bootstrap_nodes = read_network_conn_info()?;
+    let config = Config::new(Some(root_dir.path()), None, None, timeout).await;
+    let client = Client::new(config, bootstrap_nodes, optional_keypair.clone()).await?;
 
     Ok(client)
 }
